use seed::{*, prelude::*};
use drop;


fn calc_ballistics(dist: u32, zero_dist: u32, cartridge: &str) -> String {
    match drop::load_cartridges_from_string(&CARTRIDGES.to_string()) {
        Ok(cartridges) => {
            format!("{:?}", cartridges);
            match cartridges.get(cartridge) {
                Some(cartridge) => {
                    
                    format!("{:?}", cartridge.drop_zeroed(dist, zero_dist))
                },


                None => format!("Your cartridge is not available.")

            }
        },
        Err(e) => format!("{:?}", e)
    }
}

// Model

const CARTRIDGES: &str = include_str!("../../drop/loading_data.json");

struct Model {
    pub val: i32,
    pub distance: String,
    pub zero_distance: String,
    pub cartridge: String
}

impl Default for Model {
    fn default() -> Self {
        Self {
            val: 10,
            distance: "250".to_string(),
            zero_distance: "100".to_string(),
            cartridge: ".30-06 Hammerhead 14.3g".to_string(),
        }
    }
}


// Update

#[derive(Clone)]
enum Msg {
    Distance(String),
    ZeroDistance(String),
    Cartridge(String)

}

fn update(msg: Msg, model: &mut Model, _: &mut impl Orders<Msg>) {
    match msg {
        Msg::Distance(d) => model.distance = d,
        Msg::ZeroDistance(zd) => model.zero_distance = zd,
        Msg::Cartridge(c) => model.cartridge = c
    }

}


// View

fn view(model: &Model) -> impl View<Msg> {
    div![
        attrs! {
            At::Class => "panel"
        },
        h3![
            {

            format!(
                "dist:{} zero:{} cartridge:{} ballistics:{}",
                model.distance,
                model.zero_distance,
                model.cartridge,
                calc_ballistics(model.distance.parse().unwrap_or(50), model.zero_distance.parse().unwrap_or(50), &model.cartridge)
            )
            }
        ],
        label!["Distance",
            attrs! {
                At::For => "distance"
            }
        ],
        input![
            attrs! {
                At::Value => model.distance;
                At::Id => "distance"
            },
            input_ev(Ev::Input, Msg::Distance),

        ],
        label!["Zero distance",
            attrs! {
                At::For => "zero_distance"
            }
        ],
        input![
            attrs! {
                At::Value => model.zero_distance;
                At::Id => "zero_distance"
            },
            input_ev(Ev::Input, Msg::ZeroDistance),

        ],
        select![
            attrs!{At::Value => "0"},
            option![attrs!{At::Value => ".30-06 Hammerhead 14.3g"}, ".30-06 Hammerhead 14.3g"],
            option![attrs!{At::Value => ".30-06 Hammerhead 11g"}, ".30-06 Hammerhead 11g"],
            option![attrs!{At::Value => ".300 WM Barnes VOR-TX 11g"}, ".300 WM Barnes VOR-TX 11g"],
            input_ev(Ev::Input, Msg::Cartridge)
        ]
        // button![
        //     simple_ev(Ev::Click, Msg::Increment),
            
        //     format!("Hello, World × {}", model.val)
        // ]
    ]
}

#[wasm_bindgen(start)]
pub fn render() {
    App::builder(update, view)
        .build_and_start();
}